;; Copyright 2009, 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(defun my-buffer-first-line ()
  (save-excursion
    (goto-char (point-min))
    (buffer-substring (point-min) (line-end-position))))

(compile "set -x; set -e;
  cd /down/el/ell;
  if [ -s ell ]; then mv ell ell.old; fi;
  wget -nv http://www.damtp.cam.ac.uk/user/sje30/emacs/ell -O ell
")

(let (my-packages)
  (require 'cl)
  (require 'lisp-mnt)
  (defvar my-ell-packages nil)

  (setq my-ell-packages nil)
  (with-temp-buffer
    (insert-file-contents "/down/el/ell/ell")
    (goto-char (point-min))
    (while (re-search-forward ";;; \\(.*\\) --- .*" nil t)
      (add-to-list 'my-ell-packages (match-string 1))))

  (setq my-packages
        (directory-files ".." nil "\\.el\\'"))
  (setq my-packages (set-difference my-packages
                                    '("miscbits-byte-compile.el",
                                      "miscbits-el-finder-inf.el"
                                      "miscbits-generate-finder.el"
                                      "miscbits-generate-loaddefs.el")
                                    :test 'string-equal))

  (setq my-packages (set-difference my-packages my-ell-packages
                                    :test 'string-equal))
  (setq my-packages (sort my-packages 'string-lessp))

  (switch-to-buffer "*my-ell*")
  (erase-buffer)
  (insert (format "total %d\n\n" (length my-packages)))

  (let ((date (format-time-string "%Y-%02m-%02d"))
        summary url)
    (dolist (package my-packages)
      (with-temp-buffer
        (insert-file-contents (concat "../" package))
        (goto-char (point-min))
        (setq summary (my-buffer-first-line)
              url     (lm-header "URL")))
      (insert summary "\n"
              "D: " date "\n"
              "S: " (or url "<<<NO-URL>>>") "\n"
              "A: Kevin Ryde\n"
              "\n")))
  (goto-char (point-min))))
    
