;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Look through all defcustom variables to see that for hook type all the
;; custom-options values are bound functions.  And some related checks.

(defun my-custom-option-fboundp ()
  (require 'cl)
  (let ((count 0)
        (count-hooks 0))
    (mapatoms
     (lambda (symbol)
       (when (eq 'hook (get symbol 'custom-type))
         (incf count-hooks)
         (let ((option-list (get symbol 'custom-options)))
           (when option-list
             (incf count)
             (dolist (option option-list)
               (unless (functionp option)
                 (lwarn 'custom-option-fboundp
                        :error
                        "hook variable `%s' option `%s' not bound"
                        symbol option))))))

       (when (and (not (boundp symbol))
                  (plist-member (symbol-plist symbol) 'custom-options))
         (lwarn 'custom-option-fboundp
                :error
                "unbound symbol `%s' has custom-options %S"
                symbol
                (get symbol 'custom-options)))

       (when (and (not (memq (get symbol 'custom-type)
                             '(nil hook function symbol alist string)))
                  (plist-member (symbol-plist symbol) 'custom-options))
         (lwarn 'custom-option-fboundp
                :error
                "symbol `%S' type %S has custom-options %S"
                symbol
                (get symbol 'custom-type)
                (get symbol 'custom-options)))))

    (message "checked %d customs (%d hooks)" count count-hooks)))
  

(progn
  (unless (featurep 'my-load-all)
    (load (expand-file-name "my-load-all.el")))
  (my-load-all--custom))

(my-custom-option-fboundp)

(setq debug-on-error t)
(require 'calendar)
(get 'calendar-load-hook 'custom-options)
