;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Look through all keymaps for key bindings which are undefined symbols, or
;; non-command functions.

(progn
  (require 'cl)
  (defvar my-count 0)
  (defun my-check-keymap (from keymap)
    (map-keymap (lambda (type binding)
                  (incf my-count)
                  (cond ((keymapp binding)
                         (my-check-keymap (format "%s %S" from type)
                                          binding))
                        ((eq type 'follow-link)
                         )
                        ((null binding)
                         )
                        ((symbolp binding)
                         (cond ((commandp binding)
                                ;; ok
                                )
                               ((fboundp binding)
                                (lwarn 'emacs-keymaps :error
                                       "%s %S `%s' bound but not `commandp'"
                                       from type binding))
                               (t
                                (lwarn 'emacs-keymaps :error
                                       "%s %S `%s' unbound"
                                       from type binding))))))
                keymap)))

(my-check-keymap "text-mode-map" text-mode-map)

(progn
  (setq my-count 0)
  (mapatoms
   (lambda (symbol)
     (when (and (boundp symbol) ;;  'variable-documentation)
                (keymapp (symbol-value symbol)))
       (my-check-keymap (symbol-name symbol)
                        (symbol-value symbol)))))
  my-count)

