;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Look for variables appearing as @code{} in the emacs manuals and which
;; don't have @vindex entries.



;; (eval-buffer)

;;(defvar my-manual "emacs")
;;(defvar my-manual "elisp")
(defvar my-manual "both")

(require 'cl)
(setq warning-minimum-level :debug)
(setq warning-minimum-log-level :debug)

(defun my-hash-first-entry (hashtable)
  "Return the first key from HASHTABLE.
This is done with `maphash' and so in practice the \"first\" is a
random key."
  (catch 'return
    (maphash (lambda (key value)
               (throw 'return key))
             hashtable)))

(defun my-hash-keys (hashtable)
  "Return a list of all the keys in HASHTABLE."
  (let (ret)
    (maphash (lambda (key value)
               (push key ret))
             hashtable)
    (nreverse ret)))

;;-----------------------------------------------------------------------------

(unless (equal my-manual "both")
  (require 'info-look)
  (info-lookup-reset)

  (add-to-list 'load-path "~/lisp")
  (require 'info-lookmore) ;; mine

  (info-lookmore-modify-doclist 'symbol 'emacs-lisp-mode
    (lambda (doc-list)
      (remove* (concat "\\`(" (regexp-quote my-manual) ")")
               doc-list
               :key 'car
               :test-not 'string-match))))
;; (info-lookmore->mode-value 'symbol 'emacs-lisp-mode)

(defvar my-vindex-hash
  (let ((h (make-hash-table :test 'eq)))
    (require 'info-look)
    (dolist (pair (info-lookup->completions 'symbol 'emacs-lisp-mode))
      (puthash (intern (car pair)) t h))
    (lwarn 'emacs-vindex :debug "vindex %d first `%S'"
           (hash-table-count h)
           (my-hash-first-entry h))
    h))

(defun my-autoloadp (obj)
  (and (consp obj)
       (eq (car obj) 'autoload)))
;; (defun my-symbol-autoload-function-p ()

(defun my-load-symbol-autoloads (symbol)
  (when (fboundp symbol)
    (let ((func (symbol-function symbol)))
      (when (my-autoloadp func)
        (load (nth 1 func))))))

;; (my-load-symbol-autoloads 'browse-url)
;; (symbol-function 'browse-url)
;; (featurep 'browse-url)

(let ((more t))
  (while more
    (let ((before (length features)))
      (mapc 'my-load-symbol-autoloads
            (my-hash-keys my-vindex-hash))
      (let* ((after (length features))
             (count (- after before)))
        (lwarn 'emacs-vindex :debug "vindex autoloads %d features" count)
        (setq more (/= 0 count))))))

(defvar my-undoc-hash
  (let ((h (make-hash-table :test 'eq)))
    (mapatoms
     (lambda (symbol)
       (when (and (boundp symbol) ;;  'variable-documentation)
                  (not (gethash symbol my-vindex-hash)))
         (puthash symbol t h))))
    (lwarn 'emacs-vindex :debug
           "undoc %d first `%S'"
           (hash-table-count h)
           (my-hash-first-entry h))
    h))

;; (gethash 'major-mode my-vindex-hash)
;; (gethash 'major-mode my-undoc-hash)

(defconst my-emacs-only-re
  (concat "\\`"
          (regexp-opt (mapcar 'expand-file-name
                              '("/usr/share/emacs/23.1/"
                                "/down/emacs/"
                                "~/.abbrev_defs"))
                      t)))

(progn
  (dolist (hist load-history)
    (unless (string-match my-emacs-only-re (car hist))
      (dolist (elem (cdr hist))
        (when (symbolp elem)
          (remhash elem my-undoc-hash)
          (message "exclude %s %s" elem (car hist))
          ))))
  (lwarn 'emacs-vindex :debug
         "emacs-only undoc %d first `%S'"
         (hash-table-count my-undoc-hash)
         (my-hash-first-entry my-undoc-hash)))

;; (gethash 'AltGr my-undoc-hash)

(defvar my-texis
  (set-difference (append
                   (if (equal my-manual "emacs")
                       nil
                     (file-expand-wildcards "/down/emacs/doc/lispref/*.texi"))
                   (if (equal my-manual "elisp")
                       nil
                     (file-expand-wildcards "/down/emacs/doc/emacs/*.texi")))

                  '("/down/emacs/doc/lispref/anti.texi"
                    "/down/emacs/doc/emacs/ack.texi")))

(defvar my-alist
  (let (a)
    (dolist (filename my-texis)
      (find-file filename)
      (goto-char (point-min))
      (while (re-search-forward "@code{\\([a-zA-Z0-9_][-a-zA-Z0-9_]*\\)}"
				nil t)
        (let* ((str    (match-string 1))
	       (symbol (intern-soft str)))
          (when (and symbol
		     (gethash symbol my-undoc-hash))

            (save-excursion
              (goto-char (match-beginning 1))
              (let* ((report (format "%s:%d:%d: here\n"
                                     filename
                                     (line-number-at-pos)
                                     (1+ (current-column))))
                     (elem   (or (assoc str a)
                                 (let ((new (cons str nil)))
                                   (push new a)
                                   new))))
                (push report (cdr elem)))))))
      (bury-buffer))

    (sort a (lambda (x y)
              (string-lessp (car x) (car y))))))
(length my-alist)
(car my-alist)

(progn
  (kill-buffer (get-buffer-create "xx"))
  (switch-to-buffer "xx")
  (insert (format "total %d\n" (length my-alist)))

  (dolist (elem my-alist)
    (apply 'insert
           "\n@vindex " (car elem) "\n"
           (reverse (cdr elem))))
  (goto-char (point-min))
  (compilation-mode))
