#!/usr/bin/perl -w

# Copyright 2009, 2010, 2012, 2017 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Usage: perl grep-autoload-blank.pl
#

use 5.010;
use strict;
use warnings;
use POSIX ();
use Perl6::Slurp;
use IO::Uncompress::Gunzip ;
use Iterator::Simple 'igrep';
use Iterator::Simple::Locate;

use lib::abs '.';
use MyStuff;
use MyUniqByInode;

my $verbose = 0;

my $it = Iterator::Simple::Locate->new (globs => [
                                                 # '*.el',
                                                  '*.el.gz'
                                                 ]);
$it = igrep { -e $_ } $it;
{ my $uniq = MyUniqByInode->new;
  $it = igrep { $uniq->uniq($_) } $it;
}

# my $cookie = ';;;###autoload';

while (my $filename = $it->next) {
  if ($verbose) { print "look at $filename\n"; }

  my $str = zslurp ($filename) // next;
  # print $filename,$str; exit 0;

  # blank ;;;###autoload among consecutive -- none
  #
  $str =~ /^;;;\#\#\#autoload[ \t]*(\n;;;\#\#\#autoload|[^ \t\r\n].*\n;;;\#\#\#autoload[ \t]*\n[^(])/m
    or next;

  my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $-[0]);
  print "$filename:$line:$col:\n$&\n";
}

sub zslurp {
  my ($filename) = @_;
  my $str;
  eval { $str = Perl6::Slurp::slurp(['gzip(auto)'=>1],$filename); 1 }
    or do {
      #         if ($! == POSIX::ENOENT) {
      #        }
      print "Cannot read $filename: $!\n";
      return;
    };
  return $str;


  #   my $str;
  #   if ($filename =~ /\.gz$/) {
  #     IO::Uncompress::Gunzip::gunzip ($filename => \$str)
  #         or do {
  #           print "Cannot gunzip $filename: $IO::Uncompress::Gunzip::GunzipError\n";
  #           return;
  #         };
  #
  #   } else {
  #     eval { $str = Perl6::Slurp::slurp($filename); 1 }
  #       or do {
  #         #         if ($! == POSIX::ENOENT) {
  #         #        }
  #         print "Cannot read $filename: $!\n";
  #         return;
  #       };
  #   }
}
exit 0;
