;; Copyright 2013, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(progn
  (require 'csv-mode)
  )

(progn
  (require 'modula2)
  (symbol-function 'modula-2-mode)
  (find-lisp-object-file-name 'm2-mode (symbol-function 'm2-mode))
  (get 'modula2 'custom-group)
  (find-lisp-object-file-name 'modula2))

(progn
  (defun my-custom-group-p (symbol)
    "Return non-nil if SYMBOL is a defgroup."
    (get symbol 'custom-group))
  ;; (my-custom-group-p 'modula2)

  (setq my-group-alist nil)
  (mapatoms (lambda (symbol)
              (and (my-custom-group-p symbol)
                   (push (cons symbol
                               (find-lisp-object-file-name symbol 'defgroup)))))))

                        



(progn
  (require 'cl)
  (require 'apropos)

  (defun my-minor-mode-p (symbol)
    "Return non-nil if SYMBOL is a minor mode."
    (memq symbol minor-mode-list))
  ;; (my-minor-mode-p 'which-function-mode)

  (defun my-defalias-p (symbol)
    "Return non-nil if SYMBOL is a `defalias'."
    (symbolp (symbol-function symbol)))

  (defun my-symbol-function-filename (symbol)
    "Return the filename defining SYMBOL function, or nil."
    (and (fboundp symbol)
         (find-lisp-object-file-name symbol (symbol-function symbol))))
  ;; (my-symbol-function-filename 'm2-mode)

  (defun my-custom-group-to-filename (symbol)
    "Return filename containing defgroup of SYMBOL, or nil if no such."
    (car (rassoc symbol custom-current-group-alist)))
  ;; (my-custom-group-to-filename 'modula2)

  (defun my-filename-to-custom-group (filename)
    "Return a defgroup in FILENAME, or nil if no such."
    (or (cdr (assoc filename custom-current-group-alist))
        (cdr (assoc (concat filename "c") custom-current-group-alist))))
  ;; (my-filename-to-custom-group (find-lisp-object-file-name 'm2-mode (symbol-function 'm2-mode)))


  (setq my-mode-symbols nil)
  (mapatoms (lambda (symbol)
              (and (commandp symbol)
                   (string-match "-mode\\'" (symbol-name symbol))
                   (not (apropos-macrop symbol)) ;; not macro
                   (not (string-match "\\`speechd-" (symbol-name symbol)))
                   (not (string-match "minor-mode" (symbol-name symbol)))
                   (not (string-match "\\`turn-on-" (symbol-name symbol)))
                   (not (string-match "\\`erc-" (symbol-name symbol)))
                   (not (string-match "-for-org-mode\\'" (symbol-name symbol)))
                   (not (string-match "-in-major-mode\\'" (symbol-name symbol)))

                   (push symbol my-mode-symbols))))


  (dolist (symbol '(org-fontify-like-in-org-mode
                    w3m-fb-mode
                    frame-set-background-mode
                    allout-widgets-mode
                    viper-mode
                    erc-nickserv-identify-mode
                    erc-toggle-channel-mode
                    set-scroll-bar-mode
                    magit-stgit-mode
                    magit-svn-mode
                    auto-revert-tail-mode ;; only visiting a file
                    cc-choose-style-for-mode
                    deb-view-mode
                    describe-mode-local-bindings-in-mode

                    database-mode
                    database-edit-mode
                    database-view-mode
                    database-summary-mode ;; dodgy
                    db-view-mode
                    db-edit-mode

                    inferior-python-mode ;; slightly not working
                    py-toggle-autopair-mode

                    org-agenda-log-mode ;; in an org-mode buffer
                    wdired-change-to-dired-mode
                    wdired-change-to-wdired-mode
                    help-fns--parent-mode
                    archive-calc-mode archive-int-to-mode

                    array-mode ;; various read-number
                    c-find-assignment-for-mode
                    ctypes-define-type-in-mode
                    ctypes-edit-types-in-mode
                    ctypes-get-auto-mode
                    ctypes-get-mode
                    ctypes-remove-type-in-mode
                    ctypes-string-to-mode
                    cua-selection-mode
                    customize-mode
                    develock-mode ;; takes an arg
                    dframe-frame-mode

                    edmacro-mode
                    erc-cmd-MODE
                    emacs-lock--set-mode
                    erc-invite-only-mode
                    erc-netsplit-MODE
                    forms-mode
                    au\mix-mode ;; \ to hide from grep
                    bookmark-edit-annotation-mode
                    ffap-c-mode
                    ffap-el-mode
                    ffap-fortran-mode
                    ffap-latex-mode
                    ffap-ro-mode-mode
                    ffap-tex-mode
                    eshell-pred-file-mode

                    *table--cell-describe-mode

                    ;; not without program
                    speechd-speak-mode
                    speechd-set-capital-character-mode
                    speechd-set-output-module
                    speechd-set-punctuation-mode

                    crosshairs-mode
                    ;; Buffer-menu-mode
                    ;; Custom-mode
                    ))
    (my-remove-from-list 'my-mode-symbols symbol))

  (with-current-buffer (get-buffer-create "*grep-major-modes*")
    (erase-buffer)
    (insert (format "modes %S\n\n" (length my-mode-symbols))))

  (load (expand-file-name "my-load-all.el"))
  (dolist (symbol my-mode-symbols)
    (my-load-all--one-function symbol))

  (setq my-mode-symbols (remove-if 'my-defalias-p my-mode-symbols))
  (setq my-mode-symbols (set-difference my-mode-symbols minor-mode-list))

  (setq my-mode-symbols
        (sort* my-mode-symbols 'string< :key 'symbol-name))


  (dolist (symbol my-mode-symbols)
    (message "%S" symbol)
    (with-temp-buffer

      (setq dictionary-window-configuration (current-window-configuration))
      (setq dictionary-selected-window (selected-window))

      ;; (setq comment-start nil)
      ;; (funcall symbol)
      ;; (when (and comment-start
      ;;            (not (local-variable-p 'comment-start)))
      ;;   (error "%S" symbol))


      ;; listing comment-start-skip values
      ;; (funcall symbol)
      ;; (when (and comment-start-skip
      ;;            (string-match "\\\\(" comment-start-skip)))
      ;;   (with-current-buffer "*grep-major-modes*"
      ;;     (insert (format "%S comment-start-skip is %S\n"
      ;;                     symbol comment-start-skip))))


      ;; checking `customize-mode'
      ;; (unless (custom-group-of-mode symbol)
      ;;   (let ((group (my-filename-to-custom-group (my-symbol-function-filename symbol))))
      ;;     (when group
      ;;       (with-current-buffer "*grep-major-modes*"
      ;;         (insert (format "no (custom-group-of-mode '%28S)  %S\n"
      ;;                         symbol group))))))

      ;; checking `-mode-map' appears in docstring
      (let ((docstring (documentation symbol
                                      t))) ;; raw
        (cond ((not docstring)
               (with-current-buffer "*grep-major-modes*"
                     (insert (format "%S no docstring\n"
                                       symbol))))
              ((not (string-match "\\\\{" docstring))
                    (with-current-buffer "*grep-major-modes*"
                          (insert (format "%S no keymap in docstring\n"
                                            symbol)))))))

      ))

  (switch-to-buffer "*grep-major-modes*"))


;; database-summary-mode
;; inferior-python-mode
;; develock-mode


;; m2-mode                 :group 'modula2
;; sql-mode                :group 'SQL  
;; display-time-world-mode :group 'display-time

