;; Copyright 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(progn
  (require 'cl)
  (require 'apropos)

  (defun my-string-first-line (str)
    (or (string-match ".*$" str) (error "oops, no match my-string-first-line"))
    (match-string 0 str))

  (defun my-minor-mode-p (symbol)
    "Return non-nil if SYMBOL is a minor mode."
    (memq symbol minor-mode-list))
  ;; (my-minor-mode-p 'which-function-mode)

  (defun my-defalias-p (symbol)
    "Return non-nil if SYMBOL is a `defalias'."
    (symbolp (symbol-function symbol)))

  (defun my-symbol-function-filename (symbol)
    "Return the filename defining SYMBOL function, or nil."
    (and (fboundp symbol)
         (find-lisp-object-file-name symbol (symbol-function symbol))))
  ;; (my-symbol-function-filename 'm2-mode)

  (setq my-mode-symbols nil)
  (mapatoms (lambda (symbol)
              (and (commandp symbol)
                   (not (my-minor-mode-p symbol))
                   (string-match "[Mm]inor"
                                 (my-string-first-line (or (documentation symbol) "")))

                   ;; (string-match "-mode\\'" (symbol-name symbol))
                   ;; (not (apropos-macrop symbol)) ;; not macro
                   (not (string-match "\\`turn-on-" (symbol-name symbol)))
                   ;; (not (string-match "\\`erc-" (symbol-name symbol)))
                   ;; (not (string-match "-for-org-mode\\'" (symbol-name symbol)))
                   ;; (not (string-match "-in-major-mode\\'" (symbol-name symbol)))

                   (push symbol my-mode-symbols))))

  (with-current-buffer (get-buffer-create "*grep-minor-modes*")
    (erase-buffer)
    (insert (format "modes %S\n\n" (length my-mode-symbols))))

  (load (expand-file-name "my-load-all.el"))
  (dolist (symbol my-mode-symbols)
    (my-load-all--one-function symbol))

  (setq my-mode-symbols
        (sort* my-mode-symbols 'string< :key 'symbol-name))

  (dolist (symbol my-mode-symbols)
    (let ((docstring (or (documentation symbol) "")))
      (with-current-buffer "*grep-minor-modes*"
        (insert (format "%S  %S\n" symbol (my-string-first-line docstring))))
      ))

  (switch-to-buffer "*grep-minor-modes*"))


(let ((lst (mapcar 'car minor-mode-alist)))
  (require 'cl)
  (setq lst (set-difference lst minor-mode-list))
  (setq lst (sort* lst 'string< :key 'symbol-name))
  (dolist (symbol lst)
    (insert (format "%s\n" symbol))))
2C-mode
compilation-in-progress
defining-kbd-macro
dictionary-tooltip-mode
filladapt-mode
folding-mode
haskell-doc-mode
hide-ifdef-hiding
highlight-current-line-minor-mode
isearch-mode
mmm-mode
quilt-mode
rcirc-ignore-buffer-activity-flag
rcirc-low-priority-flag
rcirc-omit-mode
w3m-bookmark-mode
w3m-form-input-textarea-mode
w3m-minor-mode
