;; Copyright 2009, 2010, 2011, 2013 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (bufname "*t-copyright-both-ends--temp*"))
    (setq copyright-query nil)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(copyright-both-ends-update))


    ;;-------------------------------------------------------------------------
    ;; copyright-both-ends-update -- one

    (switch-to-buffer bufname)
    (insert "Copy" "right (C) 2000 Foo\n") ;; disguised from copyright-update
    (let ((old (buffer-string)))
      (copyright-both-ends-update)

      (when (fboundp 'copyright-update)
        (let ((new (buffer-string)))
          (when (equal old new)
            (error "one copyright not updated")))))
    (kill-buffer bufname)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; copyright-both-ends-update -- two

    (switch-to-buffer bufname)
    (insert "Copy" "right (C) 2000 Foo\n" ;; disguised from copyright-update
            "\nsome middle text\n\n"
            "Copy" "right (C) 2000 Foo\n")
    (let ((old-1 (progn
                   (goto-char (point-min))
                   (buffer-substring (point) (line-end-position))))
          (old-2 (progn
                   (goto-char (point-max))
                   (forward-line -1)
                   (buffer-substring (point) (line-end-position)))))
      (copyright-both-ends-update)

      (when (boundp 'copyright-at-end-flag)
        (let ((new-1 (progn
                       (goto-char (point-min))
                       (buffer-substring (point) (line-end-position))))
              (new-2 (progn
                       (goto-char (point-max))
                       (forward-line -1)
                       (buffer-substring (point) (line-end-position)))))
          (when (or (equal old-1 new-1)
                    (equal old-2 new-2))
            (error "two copyrights not updated")))))
    (kill-buffer bufname)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'copyright-both-ends)
    (unload-feature 'copyright-both-ends t)
    (require 'copyright-both-ends)
    (unload-feature 'copyright-both-ends t)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))
