;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  (let ((count 0)
        (tempbuf "*t-perl-use-utf8-coding--temp*"))
    (switch-to-buffer tempbuf)

    (setq-default case-fold-search t)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'perl-use-utf8-coding)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (if (not (or (boundp 'auto-coding-functions)
                 (fboundp 'set-auto-coding)))
        (message "  skip, not a known coding setup style")

      (mapcar (lambda (elem)
                (let ((want (car elem))
                      (str  (cadr elem)))

                  (unless (coding-system-p 'utf-8)
                    (setq want nil))

                  (erase-buffer)
                  (insert str)
                  (goto-char (point-min))
                  (let ((got (perl-use-utf8-auto-coding-function
                              (- (point-max) (point-min)))))
                    (unless (equal got want)
                      (error "got %s want %s" got want)))
                  (setq count (1+ count))))

              '((utf-8 "use utf8;")
                (utf-8 "use utf8;  ")
                (utf-8 "use utf8;\n")
                (utf-8 "use utf8;\r")
                (utf-8 "blah;\nuse utf8;\t\t\t")
                (utf-8 "blah;\nuse utf8;\t# comment")
                (utf-8 "blah;\nuse utf8; \t # comment\n")

                (nil   "use\tutf8;")
                (nil   "use UTF8;")
                (nil   "use utf8 ;")
                (nil   "use utf8\t;")
                (nil   "use utf8\t\t\t")
                (nil   "use utf8")
                (nil   "    use utf8;")
                (nil   "Use utf8 in something something")
                (nil   "could use utf8; or maybe not")))

      ;;-----------------------------------------------------------------------
      ;; unload-feature

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)

      (let ((feature 'perl-use-utf8-coding))
        (when (and (fboundp 'set-auto-coding) ;; not in xemacs
                   (not (boundp 'auto-coding-functions))) ;; not in emacs22 up
          ;; emacs21
          (unless (my-advice-by-name feature)
            (error "oops, didn't find any advice")))

        (unload-feature feature t)

        (when (and (fboundp 'set-auto-coding) ;; not in xemacs
                   (not (boundp 'auto-coding-functions))) ;; not in emacs22 up
          ;; emacs21
          (let ((lst (my-advice-by-name feature)))
            (when lst
              (error "unload-feature leftover advice: %S" lst))))
        (setq count (1+ count)))


      ;;-----------------------------------------------------------------------
      ;; unload-feature -- with advice unloaded first

      (let ((feature 'perl-use-utf8-coding))
        (if (featurep feature)
            (unload-feature feature t))
        (require feature)
        ;; if 'advice can be unloaded (no dependency expressed on it), then
        ;; see FEATURE unloads successfully too
        (when (condition-case nil
                  (progn (unload-feature 'advice) t)
                (error nil))
          (unload-feature feature t)))
      (setq count (1+ count))

      ;;-----------------------------------------------------------------------
      (bury-buffer)
      (message "  %d tests" count)
      count)))
