;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-overlay-is-texinfo-warn-p (overlay)
  (eq 'texinfo-warn (overlay-get overlay 'face)))

(defun my-texinfo-warn-overlay-at-p (pos)
  (memq t (mapcar 'my-overlay-is-texinfo-warn-p (overlays-at pos))))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count   0)
        (tempbuf "*t-texinfo-warn--temp*"))
    (switch-to-buffer tempbuf)
    (erase-buffer)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(texinfo-warn-enable
              texinfo-warn-mode))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((texinfo-mode-hook texinfo-warn-enable)))

    ;;-------------------------------------------------------------------------
    ;; texinfo-warn-enable -- buffer-local after-change-functions

    (setq after-change-functions nil)
    (let ((want after-change-functions))
      (with-temp-buffer
        (texinfo-warn-enable))
      (let ((got after-change-functions))
        (unless (equal got want)
          (error "texinfo-warn-enable changes global after-change-functions: got %S want %S" got want)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; autoloaded mode func

    (texinfo-warn-mode 1)
    (unless (equal t texinfo-warn-mode)
      (error "texinfo-warn-mode doesn't set texinfo-warn-mode variable"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; texinfo-warn-disable

    (texinfo-warn-disable)
    (unless (equal nil texinfo-warn-mode)
      (error "texinfo-warn-disable doesn't set texinfo-warn-mode variable"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; texinfo-warn-enable

    (texinfo-warn-enable)
    (unless (equal t texinfo-warn-mode)
      (error "texinfo-warn-enable doesn't set texinfo-warn-mode variable"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; overlay -- initial

    (texinfo-warn-disable)
    (erase-buffer)
    (insert "\t")
    (texinfo-warn-enable)
    (unless (my-texinfo-warn-overlay-at-p (point-min))
      (error "no overlay on tab initial enable"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; overlay -- on insert

    (texinfo-warn-disable)
    (erase-buffer)
    (texinfo-warn-enable)
    (insert "\t")
    (unless (my-texinfo-warn-overlay-at-p (point-min))
      (error "no overlay on tab insert"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (erase-buffer)
      (insert "\t")
      (texinfo-warn-enable)
      (unless (my-texinfo-warn-overlay-at-p (point-min))
        (error "oops, no overlay on tab"))

      (unload-feature 'texinfo-warn t)

      (when (my-texinfo-warn-overlay-at-p (point-min))
        (error "unload-feature leaves overlay on tab"))
      (when (and (listp after-change-functions)
                 (memq 'texinfo-warn-after-change after-change-functions))
        (error "unload-feature leftover after-change-functions %S"
               after-change-functions))
      (setq count (1+ count)))

    ;; couple of cycles
    (require 'texinfo-warn)
    (unload-feature 'texinfo-warn t)
    (require 'texinfo-warn)
    (unload-feature 'texinfo-warn t)
    (setq count (1+ count))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'texinfo-warn))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'texinfo-warn)
      (kill-buffer nil))


    (bury-buffer)
    (message "  %d tests" count)
    count))
